package hu.bme.mit.mdsd.service.generator.ui;

import java.util.HashMap;
import java.util.Map;

import hu.bme.mit.mdsd.service.generator.ServiceModelGenerator;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.xtext.builder.EclipseResourceFileSystemAccess2;
import org.eclipse.xtext.builder.EclipseResourceFileSystemAccess2.IFileCallback;
import org.eclipse.xtext.generator.IFileSystemAccess;
import org.eclipse.xtext.generator.OutputConfiguration;

import service.GeneratorModel;

public class GenerateCode implements IObjectActionDelegate {

	private GeneratorModel genModel;

	@Override
	public void run(IAction action) {
		if (genModel != null) {
			Job job = new Job("Codegen") {
				@Override
				protected IStatus run(IProgressMonitor monitor) {
					try {
						String pn = genModel.getProjectName();
						IProject p = ResourcesPlugin.getWorkspace().getRoot().getProject(pn);
						if (p.exists()) {
							p.delete(true, monitor);
						} 
						p.create(monitor);
						p.open(monitor);
						EclipseResourceFileSystemAccess2 fsa = getConfiguredFileSystemAccess(p);
						fsa.setMonitor(monitor);
						ServiceModelGenerator gen = new ServiceModelGenerator();
						gen.doGenerate(genModel.eResource(), fsa);						
					} catch (Exception e) {
						e.printStackTrace();
					}
					return Status.OK_STATUS;
				}
			};
			job.schedule();
		}
	}

	@Override
	public void selectionChanged(IAction action, ISelection selection) {
		if (selection instanceof IStructuredSelection) {
			if (((IStructuredSelection) selection).getFirstElement() instanceof GeneratorModel) {
				this.genModel = (GeneratorModel) ((IStructuredSelection) selection).getFirstElement(); 
			}
		}
	}

	@Override
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {

	}
	
	protected EclipseResourceFileSystemAccess2 getConfiguredFileSystemAccess(IProject p) {
		EclipseResourceFileSystemAccess2 fsa = new EclipseResourceFileSystemAccess2();
		fsa.setProject(p);
		Map<String, OutputConfiguration> outputs = new HashMap<String, OutputConfiguration>();
		OutputConfiguration out = new OutputConfiguration(IFileSystemAccess.DEFAULT_OUTPUT);
		out.setOutputDirectory("src-gen");
		out.setCanClearOutputDirectory(true);
		out.setCleanUpDerivedResources(true);
		out.setCreateOutputDirectory(true);
		out.setOverrideExistingResources(true);
		out.setSetDerivedProperty(true);
		OutputConfiguration out2 = new OutputConfiguration("SRCGEN2");
		out2.setOutputDirectory("src-gen2");
		out2.setCanClearOutputDirectory(true);
		out2.setCleanUpDerivedResources(true);
		out2.setCreateOutputDirectory(true);
		out2.setOverrideExistingResources(true);
		out2.setSetDerivedProperty(true);
		outputs.put(IFileSystemAccess.DEFAULT_OUTPUT, out);
		outputs.put("SRCGEN2", out2);
		fsa.setOutputConfigurations(outputs);
		fsa.setPostProcessor(new IFileCallback() {
			@Override
			public boolean beforeFileDeletion(IFile file) {
				return false;
			}
			
			@Override
			public void afterFileUpdate(IFile file) {
			}
			
			@Override
			public void afterFileCreation(IFile file) {
			}
		});
		return fsa;
	}

}
